#!/usr/bin/python3
# -*- coding=utf-8 -*-
r"""

"""
import unittest


class TestImport(unittest.TestCase):
    def test_import(self):
        import configlib  # noqa


if __name__ == '__main__':
    unittest.main()
